/*
 * Copyright 2012 The Clustermeister Team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.nethad.clustermeister.integration.sc07

/**
 * Creator in separate class to prevent excessive closure-capture of the TorqueNodeProvisioner class (Error[java.io.NotSerializableException TorqueNodeProvisioner])
 */
import akka.actor.ActorRef
import akka.actor.ActorSystem
import akka.actor.Props
import akka.dispatch.Await
import akka.japi.Creator
import akka.util.Timeout
import akka.util.duration._
import com.github.nethad.clustermeister.api.Clustermeister
import com.github.nethad.clustermeister.api.impl.ClustermeisterFactory
import collection.JavaConversions._
import akka.pattern.ask
import com.typesafe.config.Config
import java.net.URL
import java.util.Enumeration
import java.util.concurrent.Callable
import com.signalcollect.nodeprovisioning.torque._
import com.signalcollect.configuration.AkkaConfig
import com.signalcollect.nodeprovisioning._
import com.signalcollect.implementations.messaging.AkkaProxy
import org.jppf.classloader.AbstractJPPFClassLoader

case class NodeControllerCreator(jobId: Any, nodeProvisionerAddress: String) extends Creator[NodeControllerActor] {
  def create: NodeControllerActor = new NodeControllerActor(jobId, nodeProvisionerAddress)
}

/**
 * Creator in separate class to prevent excessive closure-capture of the TorqueNodeProvisioner class (Error[java.io.NotSerializableException TorqueNodeProvisioner])
 */
case class NodeProvisionerCreator(numberOfNodes: Int) extends Creator[NodeProvisionerActor] {
  def create: NodeProvisionerActor = new NodeProvisionerActor(numberOfNodes)
}

class ClustermeisterNodeProvisioner extends NodeProvisioner {

  var cm: Option[Clustermeister] = None

  def getNodes: List[Node] = {
    try {
      cm = Some(ClustermeisterFactory.create)
      println("Start Clustermeister")
      if (cm.isDefined) {
        val numberOfNodes = cm.get.getAllNodes.size
        val config = AkkaConfig.get
        val system: ActorSystem = ActorSystem("NodeProvisioner", config)
        val nodeProvisionerCreator = NodeProvisionerCreator(numberOfNodes)
        val nodeProvisioner = system.actorOf(Props().withCreator(nodeProvisionerCreator.create), name = "NodeProvisioner")
        val nodeProvisionerAddress = AkkaHelper.getRemoteAddress(nodeProvisioner, system)
        implicit val timeout = new Timeout(1800 seconds)

        for (node <- cm.get.getAllNodes) {
          val actorNameFuture = node.execute(NodeControllerBootstrap(node.getID, nodeProvisionerAddress, config, false))
          println("Started node controller: " + actorNameFuture.get)
        }
        val nodesFuture = nodeProvisioner ? "GetNodes"
        val result = Await.result(nodesFuture, timeout.duration)
        val nodes: List[Node] = result.asInstanceOf[List[ActorRef]] map (AkkaProxy.newInstance[Node](_))
        nodes
      } else {
        throw new Exception("Clustermeister could not be initialized.")
      }
    } finally {
      if (cm.isDefined) {
        cm.get.shutdown
      }
    }
  }

}

case class NodeControllerBootstrap(nodeId: String, nodeProvisionerAddress: String, akkaConfig: Config, preload: Boolean) extends Callable[String] {
  
  val clazzes = Array("com.signalcollect.implementations.graph.WorkerCreator", 
                      "com.signalcollect.implementations.messaging.Command", 
                      "com.signalcollect.implementations.messaging.Request",
                      "com.signalcollect.configuration.AkkaDispatcher", 
                      "com.signalcollect.implementations.messaging.Command",
                      "com.signalcollect.implementations.messaging.Command$$anonfun$1", 
                      "com.signalcollect.configuration.Pinned", 
                      "com.signalcollect.configuration.Pinned$", 
                      "com.signalcollect.implementations.graph.DefaultGraph$$anonfun$3$$anonfun$apply$1$$anonfun$4",
                      "com.signalcollect.factory.worker.Akka",
                      "com.signalcollect.DataGraphVertex$mcVB$sp",
"com.signalcollect.EdgeId$mcBS$sp",
"com.signalcollect.EdgeId$mcZB$sp$class",
"com.signalcollect.DataGraphVertex$mcBJ$sp",
"com.signalcollect.DefaultEdge$mcIS$sp",
"com.signalcollect.DataGraphVertex$mcJD$sp",
"com.signalcollect.EdgeId$mcVJ$sp$class",
"com.signalcollect.DefaultEdge$mcCJ$sp",
"com.signalcollect.EdgeId$mcZJ$sp",
"com.signalcollect.EdgeId$mcJF$sp",
"com.signalcollect.EdgeId$mcCI$sp$class",
"com.signalcollect.DataFlowVertex$mcJB$sp",
"com.signalcollect.OnlySignalOnChangeEdge",
"com.signalcollect.DataGraphVertex$mcCD$sp",
"com.signalcollect.DataFlowVertex$mcJS$sp",
"com.signalcollect.EdgeId$mcDZ$sp$class",
"com.signalcollect.EdgeId$mcID$sp$class",
"com.signalcollect.DataGraphVertex$mcCZ$sp",
"com.signalcollect.ExecutionConfiguration$",
"com.signalcollect.DefaultEdge$mcSJ$sp",
"com.signalcollect.GraphEditor$class",
"com.signalcollect.DataGraphVertex$mcJF$sp",
"com.signalcollect.DefaultEdge$mcJC$sp",
"com.signalcollect.EdgeId$mcCC$sp$class",
"com.signalcollect.DataGraphVertex$mcFZ$sp",
"com.signalcollect.DataGraphVertex$mcBZ$sp",
"com.signalcollect.DataGraphVertex$mcZI$sp",
"com.signalcollect.factory.worker.CollectFirstAkka",
"com.signalcollect.factory.worker.CollectFirstAkka$",
"com.signalcollect.factory.worker.Akka",
"com.signalcollect.factory.worker.CollectFirstAkka$$anon$1",
"com.signalcollect.factory.worker.Akka$",
"com.signalcollect.factory.messagebus.SharedMemory",
"com.signalcollect.factory.messagebus.SharedMemory$",
"com.signalcollect.factory.storage.CachedBerkeleyDB$CachedStorage",
"com.signalcollect.factory.storage.BerkeleyDB",
"com.signalcollect.factory.storage.CompressedBerkeleyDB",
"com.signalcollect.factory.storage.CachedBerkeleyDB$",
"com.signalcollect.factory.storage.CompressedBerkeleyDB$BerkeleyDBStorage",
"com.signalcollect.factory.storage.BerkeleyDB$",
"com.signalcollect.factory.storage.CachedBerkeleyDB",
"com.signalcollect.factory.storage.CompressedBerkeleyDB$",
"com.signalcollect.factory.storage.BerkeleyDB$BerkeleyDBStorage",
"com.signalcollect.factory.storage.InMemory",
"com.signalcollect.factory.storage.InMemory$",
"com.signalcollect.ExecutionInformation",
"com.signalcollect.DefaultEdge$mcIB$sp",
"com.signalcollect.EdgeId$mcDB$sp",
"com.signalcollect.DefaultEdge$mcVF$sp",
"com.signalcollect.DefaultEdge$",
"com.signalcollect.EdgeId$mcIZ$sp$class",
"com.signalcollect.EdgeId$mcFC$sp",
"com.signalcollect.EdgeId$mcVJ$sp",
"com.signalcollect.EdgeId$mcSB$sp$class",
"com.signalcollect.DataFlowVertex$mcDD$sp",
"com.signalcollect.DefaultEdge$mcZF$sp",
"com.signalcollect.EdgeId$mcSZ$sp",
"com.signalcollect.DataGraphVertex$mcIJ$sp",
"com.signalcollect.DataGraphVertex$mcSD$sp",
"com.signalcollect.Graph",
"com.signalcollect.DataGraphVertex$mcFC$sp",
"com.signalcollect.DataGraphVertex$mcCJ$sp",
"com.signalcollect.DataGraphVertex$mcDF$sp",
"com.signalcollect.EdgeId$mcFZ$sp$class",
"com.signalcollect.DataGraphVertex$mcBB$sp",
"com.signalcollect.DataFlowVertex$mcSF$sp",
"com.signalcollect.EdgeId$mcDV$sp$class",
"com.signalcollect.EdgeId$mcCI$sp",
"com.signalcollect.DataFlowVertex$mcCD$sp",
"com.signalcollect.EdgeId$mcFJ$sp$class",
"com.signalcollect.DefaultEdge$mcDC$sp",
"com.signalcollect.EdgeId$mcDS$sp",
"com.signalcollect.DataFlowVertex$mcIF$sp",
"com.signalcollect.DataGraphVertex$mcIB$sp",
"com.signalcollect.nodeprovisioning.NodeProvisioner",
"com.signalcollect.nodeprovisioning.Node",
"com.signalcollect.nodeprovisioning.AkkaHelper",
"com.signalcollect.nodeprovisioning.local.LocalNode$$anonfun$3",
"com.signalcollect.nodeprovisioning.local.LocalNode",
"com.signalcollect.nodeprovisioning.local.LocalNode$$anonfun$1",
"com.signalcollect.nodeprovisioning.local.LocalNodeProvisioner",
"com.signalcollect.nodeprovisioning.local.LocalNode$$anonfun$2",
"com.signalcollect.nodeprovisioning.torque.TorquePriority$",
"com.signalcollect.nodeprovisioning.torque.NodeProvisionerCreator$",
"com.signalcollect.nodeprovisioning.torque.LocalHost$$anonfun$executeJobs$1$$anonfun$apply$1",
"com.signalcollect.nodeprovisioning.torque.JobExecutor$",
"com.signalcollect.nodeprovisioning.torque.TorqueNodeProvisioner$$anonfun$getNodes$1$$anonfun$2",
"com.signalcollect.nodeprovisioning.torque.TorqueNodeProvisioner",
"com.signalcollect.nodeprovisioning.torque.NodeControllerActor$$anonfun$receive$1",
"com.signalcollect.nodeprovisioning.torque.IoUtil",
"com.signalcollect.nodeprovisioning.torque.TorqueHost$",
"com.signalcollect.nodeprovisioning.torque.JobExecutor$delayedInit$body",
"com.signalcollect.nodeprovisioning.torque.TorqueJob",
"com.signalcollect.nodeprovisioning.torque.IoUtil$",
"com.signalcollect.nodeprovisioning.torque.NodeProvisionerActor",
"com.signalcollect.nodeprovisioning.torque.TorqueJobSubmitter$",
"com.signalcollect.nodeprovisioning.torque.ResultHandler",
"com.signalcollect.nodeprovisioning.torque.NodeControllerActor$$anonfun$1",
"com.signalcollect.nodeprovisioning.torque.TorqueJob$",
"com.signalcollect.nodeprovisioning.torque.TorqueNodeProvisioner$$anonfun$getNodes$1",
"com.signalcollect.nodeprovisioning.torque.NodeControllerCreator",
"com.signalcollect.nodeprovisioning.torque.JobExecutor",
"com.signalcollect.nodeprovisioning.torque.ExecutionHost",
"com.signalcollect.nodeprovisioning.torque.NodeProvisionerActor$$anonfun$receive$1",
"com.signalcollect.nodeprovisioning.torque.TorqueJobSubmitter",
"com.signalcollect.nodeprovisioning.torque.TorqueHost",
"com.signalcollect.nodeprovisioning.torque.NodeControllerActor",
"com.signalcollect.nodeprovisioning.torque.TorqueNodeProvisioner$$anonfun$1",
"com.signalcollect.nodeprovisioning.torque.NodeControllerActor$$anonfun$2",
"com.signalcollect.nodeprovisioning.torque.LocalHost",
"com.signalcollect.nodeprovisioning.torque.NodeProvisionerCreator",
"com.signalcollect.nodeprovisioning.torque.LocalHost$$anonfun$executeJobs$1",
"com.signalcollect.nodeprovisioning.torque.TorqueNodeProvisioner$$anonfun$getNodes$1$$anonfun$2$$anonfun$3",
"com.signalcollect.nodeprovisioning.torque.TorqueNodeProvisioner$$anonfun$4",
"com.signalcollect.nodeprovisioning.torque.TorquePriority",
"com.signalcollect.nodeprovisioning.torque.TorqueHost$$anonfun$executeJobs$1",
"com.signalcollect.nodeprovisioning.torque.ExecutionHost$class",
"com.signalcollect.nodeprovisioning.torque.NodeControllerCreator$",
"com.signalcollect.nodeprovisioning.AkkaHelper$",
"com.signalcollect.DataGraphVertex$mcJS$sp",
"com.signalcollect.DataFlowVertex$mcFS$sp",
"com.signalcollect.EdgeId$mcJD$sp$class",
"com.signalcollect.DefaultEdge$mcBS$sp",
"com.signalcollect.DefaultEdge$mcJF$sp",
"com.signalcollect.EdgeId$mcZI$sp",
"com.signalcollect.DataGraphVertex$mcZD$sp",
"com.signalcollect.EdgeId$mcBJ$sp",
"com.signalcollect.DefaultEdge$mcDD$sp",
"com.signalcollect.DefaultEdge$mcDI$sp",
"com.signalcollect.ExecutionStatistics",
"com.signalcollect.DataGraphVertex$mcII$sp",
"com.signalcollect.EdgeId$mcSC$sp$class",
"com.signalcollect.DataGraphVertex$mcZS$sp",
"com.signalcollect.DataFlowVertex$mcCB$sp",
"com.signalcollect.EdgeId$mcFB$sp",
"com.signalcollect.DefaultEdge$mcFC$sp",
"com.signalcollect.EdgeId$mcSZ$sp$class",
"com.signalcollect.DataFlowVertex$mcZB$sp",
"com.signalcollect.DefaultEdge$mcZS$sp",
"com.signalcollect.EdgeId$mcCB$sp$class",
"com.signalcollect.DataGraphVertex$mcIC$sp",
"com.signalcollect.DefaultEdge$mcVD$sp",
"com.signalcollect.DataGraphVertex$mcBC$sp",
"com.signalcollect.DefaultEdge$mcFZ$sp",
"com.signalcollect.EdgeId$mcJJ$sp",
"com.signalcollect.DefaultEdge$mcVS$sp",
"com.signalcollect.DefaultEdge$mcDJ$sp",
"com.signalcollect.EdgeId$mcCV$sp$class",
"com.signalcollect.EdgeId$mcZV$sp$class",
"com.signalcollect.StateForwarderEdge",
"com.signalcollect.EdgeId$mcCD$sp",
"com.signalcollect.EdgeId$mcIZ$sp",
"com.signalcollect.DataGraphVertex$mcIS$sp",
"com.signalcollect.DefaultEdge$mcIJ$sp",
"com.signalcollect.EdgeId$mcSS$sp",
"com.signalcollect.DataGraphVertex$mcJC$sp",
"com.signalcollect.DataGraphVertex$mcCS$sp",
"com.signalcollect.EdgeId$mcVZ$sp$class",
"com.signalcollect.DefaultEdge$mcSF$sp",
"com.signalcollect.DefaultEdge$mcID$sp",
"com.signalcollect.EdgeId$mcSI$sp$class",
"com.signalcollect.EdgeId$mcIS$sp$class",
"com.signalcollect.DataFlowVertex$mcBF$sp",
"com.signalcollect.DataGraphVertex$mcFF$sp",
"com.signalcollect.EdgeId$mcDF$sp$class",
"com.signalcollect.EdgeId$mcBJ$sp$class",
"com.signalcollect.DefaultEdge$mcCB$sp",
"com.signalcollect.EdgeId$mcSC$sp",
"com.signalcollect.EdgeId$mcSV$sp$class",
"com.signalcollect.EdgeId",
"com.signalcollect.DefaultEdge$mcDV$sp",
"com.signalcollect.DataFlowVertex$mcZJ$sp",
"com.signalcollect.EdgeId$mcZS$sp$class",
"com.signalcollect.DefaultEdge$mcVI$sp",
"com.signalcollect.DefaultEdge$mcIF$sp",
"com.signalcollect.EdgeId$mcCF$sp$class",
"com.signalcollect.DataGraphVertex$mcDZ$sp",
"com.signalcollect.DataGraphVertex$mcSJ$sp",
"com.signalcollect.EdgeId$mcZF$sp",
"com.signalcollect.EdgeId$mcVI$sp$class",
"com.signalcollect.DataGraphVertex$mcCV$sp",
"com.signalcollect.DataGraphVertex$mcVS$sp",
"com.signalcollect.DataFlowVertex$mcJI$sp",
"com.signalcollect.DefaultEdge$mcIZ$sp",
"com.signalcollect.EdgeId$mcCS$sp$class",
"com.signalcollect.EdgeId$mcCV$sp",
"com.signalcollect.DataFlowVertex$$anonfun$uncollectedSignals$1",
"com.signalcollect.DefaultEdge$mcCS$sp",
"com.signalcollect.EdgeId$mcZV$sp",
"com.signalcollect.DataFlowVertex$mcSI$sp",
"com.signalcollect.EdgeId$mcIB$sp",
"com.signalcollect.Vertex",
"com.signalcollect.DataFlowVertex$mcVZ$sp",
"com.signalcollect.EdgeId$mcCZ$sp$class",
"com.signalcollect.DataFlowVertex$mcSJ$sp",
"com.signalcollect.EdgeId$mcZD$sp",
"com.signalcollect.DataFlowVertex$mcBV$sp",
"com.signalcollect.DataGraphVertex$mcZV$sp",
"com.signalcollect.EdgeId$mcDS$sp$class",
"com.signalcollect.EdgeId$mcVZ$sp",
"com.signalcollect.DataGraphVertex$mcJB$sp",
"com.signalcollect.DefaultEdge$mcZC$sp",
"com.signalcollect.EdgeId$mcDC$sp",
"com.signalcollect.EdgeId$mcCD$sp$class",
"com.signalcollect.EdgeId$mcFC$sp$class",
"com.signalcollect.EdgeId$mcJC$sp",
"com.signalcollect.EdgeId$mcID$sp",
"com.signalcollect.EdgeId$mcZJ$sp$class",
"com.signalcollect.DataGraphVertex$mcIZ$sp",
"com.signalcollect.DataFlowVertex$mcVI$sp",
"com.signalcollect.EdgeId$mcIV$sp$class",
"com.signalcollect.DataFlowVertex$mcBZ$sp",
"com.signalcollect.EdgeId$mcJV$sp$class",
"com.signalcollect.EdgeId$mcDB$sp$class",
"com.signalcollect.DefaultEdge$mcCC$sp",
"com.signalcollect.configuration.GraphConfiguration",
"com.signalcollect.configuration.LoggingLevel",
"com.signalcollect.configuration.Pinned",
"com.signalcollect.configuration.Pinned$",
"com.signalcollect.configuration.GraphConfiguration$$anonfun$init$default$3$1",
"com.signalcollect.configuration.GraphConfiguration$",
"com.signalcollect.configuration.EventBased$",
"com.signalcollect.configuration.AkkaDispatcher",
"com.signalcollect.configuration.EventBased",
"com.signalcollect.configuration.LoggingLevel$",
"com.signalcollect.configuration.ExecutionMode",
"com.signalcollect.configuration.ExecutionMode$",
"com.signalcollect.configuration.AkkaConfig",
"com.signalcollect.configuration.GraphConfiguration$$anonfun$apply$default$3$1",
"com.signalcollect.configuration.TerminationReason",
"com.signalcollect.configuration.TerminationReason$",
"com.signalcollect.configuration.AkkaConfig$",
"com.signalcollect.EdgeId$mcJI$sp$class",
"com.signalcollect.DataFlowVertex$mcIZ$sp",
"com.signalcollect.EdgeId$mcBZ$sp$class",
"com.signalcollect.EdgeId$mcBF$sp",
"com.signalcollect.DefaultEdge$mcZB$sp",
"com.signalcollect.EdgeId$mcVB$sp",
"com.signalcollect.DataFlowVertex$mcSV$sp",
"com.signalcollect.EdgeId$mcSF$sp",
"com.signalcollect.EdgeId$mcVV$sp",
"com.signalcollect.DataGraphVertex$mcDS$sp",
"com.signalcollect.EdgeId$mcJF$sp$class",
"com.signalcollect.DataFlowVertex$mcJJ$sp",
"com.signalcollect.DataGraphVertex$mcFI$sp",
"com.signalcollect.DataFlowVertex$mcIC$sp",
"com.signalcollect.DefaultEdge$mcFI$sp",
"com.signalcollect.DataGraphVertex$mcFB$sp",
"com.signalcollect.DataFlowVertex$mcDS$sp",
"com.signalcollect.DataFlowVertex$mcSS$sp",
"com.signalcollect.EdgeId$mcVI$sp",
"com.signalcollect.DefaultEdge$mcSS$sp",
"com.signalcollect.DataGraphVertex$mcSS$sp",
"com.signalcollect.DataFlowVertex$mcSB$sp",
"com.signalcollect.EdgeId$mcCC$sp",
"com.signalcollect.DefaultEdge$mcCZ$sp",
"com.signalcollect.DefaultEdge$mcSI$sp",
"com.signalcollect.EdgeId$mcDJ$sp",
"com.signalcollect.DataGraphVertex$mcCB$sp",
"com.signalcollect.DataFlowVertex$mcZV$sp",
"com.signalcollect.DefaultEdge$mcDB$sp",
"com.signalcollect.EdgeId$mcJJ$sp$class",
"com.signalcollect.DefaultEdge$mcZJ$sp",
"com.signalcollect.DefaultEdge$mcFS$sp",
"com.signalcollect.EdgeId$mcVC$sp",
"com.signalcollect.DataFlowVertex$mcIS$sp",
"com.signalcollect.DataFlowVertex$mcDJ$sp",
"com.signalcollect.DataGraphVertex$mcID$sp",
"com.signalcollect.ExecutionInformation$",
"com.signalcollect.DefaultEdge$mcZV$sp",
"com.signalcollect.DataFlowVertex$mcII$sp",
"com.signalcollect.DataGraphVertex$mcZF$sp",
"com.signalcollect.DataFlowVertex$mcDC$sp",
"com.signalcollect.DataFlowVertex$mcZS$sp",
"com.signalcollect.DataFlowVertex$mcSC$sp",
"com.signalcollect.DataFlowVertex$mcCC$sp",
"com.signalcollect.DataGraphVertex$mcCI$sp",
"com.signalcollect.EdgeId$mcDJ$sp$class",
"com.signalcollect.util.akka.ActorSystemRegistry$",
"com.signalcollect.util.akka.ActorSystemRegistry",
"com.signalcollect.util.collections.Filter",
"com.signalcollect.util.collections.Filter$",
"com.signalcollect.DataFlowVertex$mcFZ$sp",
"com.signalcollect.EdgeId$mcJB$sp$class",
"com.signalcollect.EdgeId$mcFB$sp$class",
"com.signalcollect.EdgeId$mcCS$sp",
"com.signalcollect.DataGraphVertex$mcSB$sp",
"com.signalcollect.EdgeId$mcBC$sp$class",
"com.signalcollect.DataGraphVertex$$anonfun$getVertexIdsOfPredecessors$1",
"com.signalcollect.DefaultEdge$mcJZ$sp",
"com.signalcollect.DataGraphVertex$mcZB$sp",
"com.signalcollect.GraphEditor",
"com.signalcollect.DataFlowVertex$mcIB$sp",
"com.signalcollect.EdgeId$mcFI$sp$class",
"com.signalcollect.DefaultEdge$mcDF$sp",
"com.signalcollect.EdgeId$mcBI$sp",
"com.signalcollect.EdgeId$mcSF$sp$class",
"com.signalcollect.EdgeId$mcBV$sp$class",
"com.signalcollect.DefaultEdge$mcFV$sp",
"com.signalcollect.EdgeId$mcFJ$sp",
"com.signalcollect.DefaultEdge$mcSD$sp",
"com.signalcollect.EdgeId$mcJV$sp",
"com.signalcollect.DefaultEdge$mcDZ$sp",
"com.signalcollect.DataFlowVertex$mcCS$sp",
"com.signalcollect.DefaultEdge$mcCI$sp",
"com.signalcollect.StateExtractor",
"com.signalcollect.DataGraphVertex$mcZC$sp",
"com.signalcollect.EdgeId$mcFS$sp$class",
"com.signalcollect.EdgeId$mcZZ$sp",
"com.signalcollect.DefaultEdge$mcCV$sp",
"com.signalcollect.EdgeId$mcSD$sp",
"com.signalcollect.GraphBuilder",
"com.signalcollect.DefaultEdge$mcCF$sp",
"com.signalcollect.EdgeId$mcZI$sp$class",
"com.signalcollect.EdgeId$mcDZ$sp",
"com.signalcollect.EdgeId$mcVD$sp",
"com.signalcollect.EdgeId$mcCB$sp",
"com.signalcollect.DataFlowVertex$mcBD$sp",
"com.signalcollect.DataFlowVertex$mcFV$sp",
"com.signalcollect.DataGraphVertex$mcCF$sp",
"com.signalcollect.DefaultEdge$mcBB$sp",
"com.signalcollect.DataGraphVertex$mcVF$sp",
"com.signalcollect.EdgeId$mcIS$sp",
"com.signalcollect.EdgeId$mcCJ$sp",
"com.signalcollect.DataGraphVertex$mcBS$sp",
"com.signalcollect.DataFlowVertex$mcCI$sp",
"com.signalcollect.DataFlowVertex$mcVB$sp",
"com.signalcollect.EdgeId$mcIF$sp",
"com.signalcollect.EdgeId$mcIV$sp",
"com.signalcollect.DataFlowVertex$mcIJ$sp",
"com.signalcollect.DataFlowVertex$mcVJ$sp",
"com.signalcollect.DataFlowVertex$mcFB$sp",
"com.signalcollect.EdgeId$mcCZ$sp",
"com.signalcollect.GlobalTerminationCondition$",
"com.signalcollect.EdgeId$mcIC$sp",
"com.signalcollect.EdgeId$mcCJ$sp$class",
"com.signalcollect.EdgeId$mcVD$sp$class",
"com.signalcollect.EdgeId$mcJC$sp$class",
"com.signalcollect.EdgeId$mcZC$sp",
"com.signalcollect.DefaultEdge$mcJD$sp",
"com.signalcollect.EdgeId$mcFD$sp",
"com.signalcollect.DefaultEdge$mcCD$sp",
"com.signalcollect.DataGraphVertex$mcCC$sp",
"com.signalcollect.DefaultEdge$mcJB$sp",
"com.signalcollect.EdgeId$mcJZ$sp$class",
"com.signalcollect.DataGraphVertex$mcFV$sp",
"com.signalcollect.EdgeId$class",
"com.signalcollect.DataFlowVertex$mcBB$sp",
"com.signalcollect.EdgeId$mcFS$sp",
"com.signalcollect.DefaultEdge$mcFD$sp",
"com.signalcollect.DataFlowVertex$mcJV$sp",
"com.signalcollect.EdgeId$mcVC$sp$class",
"com.signalcollect.DefaultEdge$mcVZ$sp",
"com.signalcollect.DefaultEdge$mcBZ$sp",
"com.signalcollect.DefaultEdge$mcVC$sp",
"com.signalcollect.EdgeId$mcBV$sp",
"com.signalcollect.DataFlowVertex$mcSD$sp",
"com.signalcollect.EdgeId$mcFI$sp",
"com.signalcollect.DataFlowVertex$mcCZ$sp",
"com.signalcollect.DataFlowVertex$mcDB$sp",
"com.signalcollect.DataGraphVertex$mcDJ$sp",
"com.signalcollect.EdgeId$mcFD$sp$class",
"com.signalcollect.DataFlowVertex$mcZD$sp",
"com.signalcollect.CountVertices",
"com.signalcollect.ExecutionConfiguration",
"com.signalcollect.EdgeId$mcSD$sp$class",
"com.signalcollect.EdgeId$mcDD$sp",
"com.signalcollect.DataGraphVertex$mcDD$sp",
"com.signalcollect.DefaultEdge$mcIV$sp",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$4",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$1",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$recalculateScores$1",
"com.signalcollect.implementations.worker.ImmediateCollectScheduler$class",
"com.signalcollect.implementations.worker.ImmediateCollectScheduler",
"com.signalcollect.implementations.worker.WorkerOperationCounters$",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$signalStep$1",
"com.signalcollect.implementations.worker.WorkerOperationCounters",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$executeCollectOperationOfVertex$1",
"com.signalcollect.implementations.worker.ImmediateCollectScheduler$$anonfun$1",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$addPatternEdge$1",
"com.signalcollect.implementations.worker.ImmediateCollectScheduler$$anonfun$2",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$2",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$scheduleOperations$2",
"com.signalcollect.implementations.worker.AkkaWorker",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$3",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$aggregate$1",
"com.signalcollect.implementations.worker.ImmediateCollectScheduler$$anonfun$scheduleOperations$1",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$receive$1",
"com.signalcollect.implementations.worker.WorkerGraphEditor",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$scheduleOperations$3",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$collectStep$1",
"com.signalcollect.implementations.worker.AkkaWorker$$anonfun$scheduleOperations$1",
"com.signalcollect.implementations.logging.DefaultLogger$$anonfun$receive$1",
"com.signalcollect.implementations.logging.DefaultLogger$",
"com.signalcollect.implementations.logging.DefaultLogger$$anonfun$init$default$1$1",
"com.signalcollect.implementations.logging.DefaultLogger",
"com.signalcollect.implementations.storage.SumSignals$$anonfun$vertexSignalFactory$3",
"com.signalcollect.implementations.storage.DefaultStorage",
"com.signalcollect.implementations.storage.MostFrequentlyUsedCache",
"com.signalcollect.implementations.storage.MostFrequentlyUsedCache$$anonfun$vertexStoreFactory$2",
"com.signalcollect.implementations.storage.BerkDBJE$class",
"com.signalcollect.implementations.storage.LRUVertexCache$",
"com.signalcollect.implementations.storage.LRUMap",
"com.signalcollect.implementations.storage.LRUMap$$anonfun$applyFunction$1",
"com.signalcollect.implementations.storage.MinimalPath$class",
"com.signalcollect.implementations.storage.BerkeleyDBStorage",
"com.signalcollect.implementations.storage.Vertex2EntityAdapter",
"com.signalcollect.implementations.storage.InMemoryStorage",
"com.signalcollect.implementations.storage.BerkDBJE",
"com.signalcollect.implementations.storage.BerkeleyDBStorage$",
"com.signalcollect.implementations.storage.LRUVertexCache",
"com.signalcollect.implementations.storage.InMemoryVertexSignalBuffer",
"com.signalcollect.implementations.storage.ReducingVertexSignalBuffer$$anonfun$foreach$default$3$1",
"com.signalcollect.implementations.storage.MinimalPath$$anonfun$vertexSignalFactory$2",
"com.signalcollect.implementations.storage.MinimalPath$$anonfun$vertexSignalFactory$1",
"com.signalcollect.implementations.storage.InMemoryVertexSignalBuffer$$anonfun$foreach$default$3$1",
"com.signalcollect.implementations.storage.ScoredVertexCache$InMemoryCache",
"com.signalcollect.implementations.storage.ScoredVertexCache$$anonfun$foreach$1",
"com.signalcollect.implementations.storage.MostFrequentlyUsedCache$class",
"com.signalcollect.implementations.storage.MinimalPath",
"com.signalcollect.implementations.storage.CachedBerkeley$$anonfun$vertexStoreFactory$1",
"com.signalcollect.implementations.storage.ScoredVertexCache$",
"com.signalcollect.implementations.storage.CachedBerkeley$class",
"com.signalcollect.implementations.storage.SumSignals$$anonfun$vertexSignalFactory$4",
"com.signalcollect.implementations.storage.CachedBerkeley",
"com.signalcollect.implementations.storage.SumSignals",
"com.signalcollect.implementations.storage.InMemoryVertexIdSet",
"com.signalcollect.implementations.storage.ScoredVertexCache",
"com.signalcollect.implementations.storage.ReducingVertexSignalBuffer",
"com.signalcollect.implementations.storage.SumSignals$class",
"com.signalcollect.implementations.storage.BerkeleyDBStorage$$anonfun$cleanUp$1",
"com.signalcollect.implementations.storage.MostFrequentlyUsedCache$$anonfun$vertexStoreFactory$1",
"com.signalcollect.implementations.coordinator.DefaultCoordinator$$anonfun$isIdle$3",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$pauseComputation$1",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$signalStep$1",
"com.signalcollect.implementations.coordinator.DefaultCoordinator",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$1",
"com.signalcollect.implementations.coordinator.DefaultCoordinator$$anonfun$onIdle$1",
"com.signalcollect.implementations.coordinator.DefaultCoordinator$$anonfun$messagesReceivedByWorkers$1",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$removeVertices$1",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$setUndeliverableSignalHandler$1",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$startComputation$1",
"com.signalcollect.implementations.coordinator.DefaultCoordinator$$anonfun$messagesSentByWorkers$1",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$collectStep$1",
"com.signalcollect.implementations.coordinator.DefaultCoordinator$$anonfun$onIdle$2",
"com.signalcollect.implementations.coordinator.OnIdle$",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$foreachVertex$1",
"com.signalcollect.implementations.coordinator.DefaultCoordinator$$anonfun$isIdle$1",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$shutdown$1",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$addPatternEdge$1",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$recalculateScores$1",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$aggregate$1",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$setSignalThreshold$1",
"com.signalcollect.implementations.coordinator.WorkerApi",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$getWorkerStatistics$2",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$getIndividualWorkerStatistics$1",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$getWorkerStatistics$1",
"com.signalcollect.implementations.coordinator.IsIdle",
"com.signalcollect.implementations.coordinator.DefaultCoordinator$$anonfun$receive$1",
"com.signalcollect.implementations.coordinator.IsIdle$",
"com.signalcollect.implementations.coordinator.OnIdle",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$setCollectThreshold$1",
"com.signalcollect.implementations.coordinator.DefaultCoordinator$$anonfun$isIdle$2",
"com.signalcollect.implementations.coordinator.WorkerApi$$anonfun$collectStep$2",
"com.signalcollect.implementations.messaging.AkkaProxy$$anonfun$invoke$1",
"com.signalcollect.implementations.messaging.AkkaProxy$",
"com.signalcollect.implementations.messaging.DefaultMessageBus$$anonfun$1",
"com.signalcollect.implementations.messaging.Request$",
"com.signalcollect.implementations.messaging.DefaultMessageBus",
"com.signalcollect.implementations.messaging.Command",
"com.signalcollect.implementations.messaging.DefaultMessageBus$$anonfun$7",
"com.signalcollect.implementations.messaging.DefaultMessageBus$$anonfun$2",
"com.signalcollect.implementations.messaging.DefaultMessageBus$$anonfun$4",
"com.signalcollect.implementations.messaging.Request",
"com.signalcollect.implementations.messaging.DefaultMessageBus$$anonfun$5",
"com.signalcollect.implementations.messaging.DefaultVertexToWorkerMapper",
"com.signalcollect.implementations.messaging.DefaultMessageBus$$anonfun$6",
"com.signalcollect.implementations.messaging.AkkaProxy",
"com.signalcollect.implementations.messaging.DefaultMessageBus$$anonfun$sendToWorkers$1",
"com.signalcollect.implementations.messaging.Command$",
"com.signalcollect.implementations.messaging.DefaultMessageBus$$anonfun$workerProxies$1",
"com.signalcollect.implementations.messaging.DefaultMessageBus$$anonfun$3",
"com.signalcollect.implementations.messaging.Command$$anonfun$1",
"com.signalcollect.implementations.serialization.CompressingSerializer$class",
"com.signalcollect.implementations.serialization.DefaultSerializer",
"com.signalcollect.implementations.serialization.CompressingSerializer$",
"com.signalcollect.implementations.serialization.DefaultSerializer$class",
"com.signalcollect.implementations.serialization.DefaultSerializer$",
"com.signalcollect.implementations.serialization.CompressingSerializer",
"com.signalcollect.implementations.serialization.RandomString",
"com.signalcollect.implementations.serialization.CompressedSerialization",
"com.signalcollect.implementations.serialization.CompressedSerialization$class",
"com.signalcollect.implementations.serialization.RandomString$",
"com.signalcollect.implementations.graph.VertexGraphEditor$class",
"com.signalcollect.implementations.graph.DefaultGraph",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$execute$1",
"com.signalcollect.implementations.graph.VertexDebugging",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$3$$anonfun$apply$1",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$initializeMessageBuses$1",
"com.signalcollect.implementations.graph.ResetStateAfterSignaling",
"com.signalcollect.implementations.graph.WorkerCreator$",
"com.signalcollect.implementations.graph.ResetStateAfterSignaling$class",
"com.signalcollect.implementations.graph.AbstractVertex$$anonfun$removeAllOutgoingEdges$1",
"com.signalcollect.implementations.graph.AbstractVertex$$anonfun$getVertexIdsOfSuccessors$1",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$execute$6",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$3",
"com.signalcollect.implementations.graph.LoggerCreator$",
"com.signalcollect.implementations.graph.VertexGraphEditor",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$execute$4",
"com.signalcollect.implementations.graph.LoggerCreator",
"com.signalcollect.implementations.graph.SumOfOutWeights",
"com.signalcollect.implementations.graph.CoordinatorCreator$",
"com.signalcollect.implementations.graph.VertexDebugging$$anonfun$executeCollectOperation$1",
"com.signalcollect.implementations.graph.DefaultGraph$",
"com.signalcollect.implementations.graph.CoordinatorCreator",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$6",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$3$$anonfun$apply$1$$anonfun$4",
"com.signalcollect.implementations.graph.WorkerCreator",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$9",
"com.signalcollect.implementations.graph.VertexDebugging$$anonfun$getVertexIdsOfPredecessors$1",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$execute$5",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$10",
"com.signalcollect.implementations.graph.AbstractVertex",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$execute$3",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$7",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$execute$2",
"com.signalcollect.implementations.graph.VertexDebugging$class",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$initializeMessageBuses$1$$anonfun$apply$2",
"com.signalcollect.implementations.graph.SumOfOutWeights$class",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$2",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$shutdown$1",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$5",
"com.signalcollect.implementations.graph.DefaultGraph$$anonfun$8",
"com.signalcollect.DataFlowVertex$mcJZ$sp",
"com.signalcollect.EdgeId$mcDD$sp$class",
"com.signalcollect.DataFlowVertex$mcFJ$sp",
"com.signalcollect.DataGraphVertex$mcSF$sp",
"com.signalcollect.EdgeId$mcJI$sp",
"com.signalcollect.EdgeId$mcCF$sp",
"com.signalcollect.EdgeId$mcDC$sp$class",
"com.signalcollect.IdStateMapAggregator",
"com.signalcollect.DefaultEdge$mcJI$sp",
"com.signalcollect.DataGraphVertex$mcDB$sp",
"com.signalcollect.DataFlowVertex",
"com.signalcollect.EdgeId$mcDV$sp",
"com.signalcollect.DataFlowVertex$mcJF$sp",
"com.signalcollect.EdgeId$mcDI$sp$class",
"com.signalcollect.EdgeId$mcII$sp",
"com.signalcollect.SumOfStates",
"com.signalcollect.EdgeId$mcSB$sp",
"com.signalcollect.EdgeId$mcZS$sp",
"com.signalcollect.DataGraphVertex$mcSV$sp",
"com.signalcollect.GraphBuilder$",
"com.signalcollect.DataGraphVertex$mcSC$sp",
"com.signalcollect.DataGraphVertex$$anonfun$signals$1",
"com.signalcollect.DataFlowVertex$mcSZ$sp",
"com.signalcollect.ProductOfStates",
"com.signalcollect.DefaultEdge$mcSC$sp",
"com.signalcollect.EdgeId$mcFF$sp$class",
"com.signalcollect.DefaultEdge$mcZZ$sp",
"com.signalcollect.DataFlowVertex$mcDF$sp",
"com.signalcollect.DefaultEdge$mcSZ$sp",
"com.signalcollect.DataGraphVertex$mcBI$sp",
"com.signalcollect.Edge$class",
"com.signalcollect.DataGraphVertex",
"com.signalcollect.DefaultEdge$mcZD$sp",
"com.signalcollect.DefaultEdge$mcBV$sp",
"com.signalcollect.DataGraphVertex$mcDI$sp",
"com.signalcollect.DataGraphVertex$mcDV$sp",
"com.signalcollect.EdgeId$mcZB$sp",
"com.signalcollect.DataGraphVertex$mcIF$sp",
"com.signalcollect.DefaultEdge$mcJS$sp",
"com.signalcollect.DataFlowVertex$mcDI$sp",
"com.signalcollect.DataFlowVertex$mcJD$sp",
"com.signalcollect.DataGraphVertex$mcSZ$sp",
"com.signalcollect.DataFlowVertex$mcBS$sp",
"com.signalcollect.EdgeId$mcBB$sp$class",
"com.signalcollect.DataFlowVertex$mcVS$sp",
"com.signalcollect.EdgeId$mcBD$sp",
"com.signalcollect.DataGraphVertex$mcIV$sp",
"com.signalcollect.GlobalTerminationCondition",
"com.signalcollect.DataFlowVertex$mcVF$sp",
"com.signalcollect.EdgeId$mcJS$sp$class",
"com.signalcollect.DataGraphVertex$mcBD$sp",
"com.signalcollect.Vertex$class",
"com.signalcollect.EdgeId$mcBF$sp$class",
"com.signalcollect.ReduceStatesOperation",
"com.signalcollect.DataGraphVertex$mcBF$sp",
"com.signalcollect.ExecutionStatistics$",
"com.signalcollect.DataGraphVertex$mcFD$sp",
"com.signalcollect.DataFlowVertex$mcFI$sp",
"com.signalcollect.DataFlowVertex$mcCF$sp",
"com.signalcollect.DataGraphVertex$mcVV$sp",
"com.signalcollect.EdgeId$mcVV$sp$class",
"com.signalcollect.EdgeId$mcIF$sp$class",
"com.signalcollect.StateForwarderEdge$",
"com.signalcollect.EdgeId$mcSJ$sp$class",
"com.signalcollect.EdgeId$mcVS$sp",
"com.signalcollect.EdgeId$mcBS$sp$class",
"com.signalcollect.DataGraphVertex$mcJI$sp",
"com.signalcollect.StateAggregator",
"com.signalcollect.DefaultEdge$mcFB$sp",
"com.signalcollect.EdgeId$mcZC$sp$class",
"com.signalcollect.DataFlowVertex$mcDZ$sp",
"com.signalcollect.EdgeId$mcIB$sp$class",
"com.signalcollect.DataFlowVertex$mcVD$sp",
"com.signalcollect.EdgeId$mcBC$sp",
"com.signalcollect.DataGraphVertex$mcVZ$sp",
"com.signalcollect.EdgeId$mcDI$sp",
"com.signalcollect.DefaultEdgeId$",
"com.signalcollect.DefaultEdge$mcII$sp",
"com.signalcollect.EdgeId$mcDF$sp",
"com.signalcollect.DefaultEdge$mcBF$sp",
"com.signalcollect.OptionalSignalEdge",
"com.signalcollect.DataFlowVertex$mcFC$sp",
"com.signalcollect.DataFlowVertex$mcZI$sp",
"com.signalcollect.DataGraphVertex$mcJZ$sp",
"com.signalcollect.EdgeId$mcZF$sp$class",
"com.signalcollect.DataGraphVertex$mcVJ$sp",
"com.signalcollect.SampleVertexIds",
"com.signalcollect.DataFlowVertex$mcIV$sp",
"com.signalcollect.EdgeId$mcBZ$sp",
"com.signalcollect.DataGraphVertex$mcJJ$sp",
"com.signalcollect.EdgeId$mcJS$sp",
"com.signalcollect.DefaultEdge$mcJV$sp",
"com.signalcollect.DataFlowVertex$$anonfun$executeCollectOperation$1",
"com.signalcollect.DataFlowVertex$mcCJ$sp",
"com.signalcollect.EdgeId$mcFV$sp$class",
"com.signalcollect.DefaultEdge$mcIC$sp",
"com.signalcollect.DataGraphVertex$mcZJ$sp",
"com.signalcollect.EdgeId$mcVB$sp$class",
"com.signalcollect.DataGraphVertex$mcBV$sp",
"com.signalcollect.EdgeId$mcZD$sp$class",
"com.signalcollect.DefaultEdge$mcSB$sp",
"com.signalcollect.DataGraphVertex$mcJV$sp",
"com.signalcollect.EdgeId$mcBD$sp$class",
"com.signalcollect.EdgeId$mcFF$sp",
"com.signalcollect.DefaultEdge$mcVJ$sp",
"com.signalcollect.EdgeId$mcSV$sp",
"com.signalcollect.DataFlowVertex$mcFD$sp",
"com.signalcollect.OnlySignalOnChangeEdge$",
"com.signalcollect.DataGraphVertex$mcFS$sp",
"com.signalcollect.DataGraphVertex$mcSI$sp",
"com.signalcollect.EdgeId$mcVF$sp$class",
"com.signalcollect.DataFlowVertex$mcZZ$sp",
"com.signalcollect.EdgeId$mcIC$sp$class",
"com.signalcollect.DataGraphVertex$mcVC$sp",
"com.signalcollect.DataGraphVertex$mcVD$sp",
"com.signalcollect.EdgeId$mcII$sp$class",
"com.signalcollect.EdgeId$mcSI$sp",
"com.signalcollect.DefaultEdge",
"com.signalcollect.DefaultEdge$mcZI$sp",
"com.signalcollect.DataGraphVertex$mcDC$sp",
"com.signalcollect.DataFlowVertex$mcCV$sp",
"com.signalcollect.DefaultEdge$mcBJ$sp",
"com.signalcollect.DefaultEdgeId",
"com.signalcollect.EdgeId$mcSJ$sp",
"com.signalcollect.DataFlowVertex$mcDV$sp",
"com.signalcollect.EdgeId$mcIJ$sp$class",
"com.signalcollect.DataFlowVertex$mcJC$sp",
"com.signalcollect.DataFlowVertex$mcBI$sp",
"com.signalcollect.DataFlowVertex$mcBC$sp",
"com.signalcollect.DataFlowVertex$mcZC$sp",
"com.signalcollect.DataFlowVertex$mcID$sp",
"com.signalcollect.DataFlowVertex$mcVC$sp",
"com.signalcollect.DataFlowVertex$mcZF$sp",
"com.signalcollect.EdgeId$mcJD$sp",
"com.signalcollect.DataFlowVertex$mcFF$sp",
"com.signalcollect.interfaces.SignalMessage$mcVJJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcIJS$sp",
"com.signalcollect.interfaces.SignalMessage$mcDJD$sp",
"com.signalcollect.interfaces.SignalMessage$mcJZB$sp",
"com.signalcollect.interfaces.SignalMessage$mcSZV$sp",
"com.signalcollect.interfaces.SignalMessage$mcJDV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVZJ$sp",
"com.signalcollect.interfaces.VertexSignalBuffer$$anonfun$foreach$default$3$1",
"com.signalcollect.interfaces.SignalMessage$mcZBV$sp",
"com.signalcollect.interfaces.SignalMessage$mcSDV$sp",
"com.signalcollect.interfaces.SignalMessage$mcCSS$sp",
"com.signalcollect.interfaces.SignalMessage$mcVZZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJJB$sp",
"com.signalcollect.interfaces.SignalMessage$mcJSC$sp",
"com.signalcollect.interfaces.SignalMessage$mcVFI$sp",
"com.signalcollect.interfaces.SignalMessage$mcJBS$sp",
"com.signalcollect.interfaces.SignalMessage$mcJVV$sp",
"com.signalcollect.interfaces.SignalMessage$mcSCF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJDS$sp",
"com.signalcollect.interfaces.SignalMessage$mcBSZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCZC$sp",
"com.signalcollect.interfaces.SignalMessage$mcBZI$sp",
"com.signalcollect.interfaces.SignalMessage$mcISZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSFZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJBJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJDC$sp",
"com.signalcollect.interfaces.SignalMessage$mcIIS$sp",
"com.signalcollect.interfaces.Factory",
"com.signalcollect.interfaces.SignalMessage$mcJCV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVZD$sp",
"com.signalcollect.interfaces.SignalMessage$mcJCF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJJV$sp",
"com.signalcollect.interfaces.Warning",
"com.signalcollect.interfaces.SignalMessage$mcIZC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDCB$sp",
"com.signalcollect.interfaces.SignalMessage$mcIFC$sp",
"com.signalcollect.interfaces.SignalMessage$mcJVS$sp",
"com.signalcollect.interfaces.SignalMessage$mcVVV$sp",
"com.signalcollect.interfaces.Config$",
"com.signalcollect.interfaces.SignalMessage$mcVDV$sp",
"com.signalcollect.interfaces.SignalMessage$mcSVZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJID$sp",
"com.signalcollect.interfaces.SignalMessage$mcFDD$sp",
"com.signalcollect.interfaces.SignalMessage$mcDBV$sp",
"com.signalcollect.interfaces.SignalMessage$mcBDC$sp",
"com.signalcollect.interfaces.SignalMessage$mcCSC$sp",
"com.signalcollect.interfaces.SignalMessage$mcJDI$sp",
"com.signalcollect.interfaces.SignalMessage$mcFJS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIJJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcISJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVBD$sp",
"com.signalcollect.interfaces.SignalMessage$mcBFF$sp",
"com.signalcollect.interfaces.SignalMessage$mcZVB$sp",
"com.signalcollect.interfaces.SignalMessage$mcZBC$sp",
"com.signalcollect.interfaces.SignalMessage$mcBSC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDJI$sp",
"com.signalcollect.interfaces.Config",
"com.signalcollect.interfaces.SignalMessage$mcSBS$sp",
"com.signalcollect.interfaces.SignalMessage$mcJZZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSVB$sp",
"com.signalcollect.interfaces.SignalMessage$mcCVV$sp",
"com.signalcollect.interfaces.SignalMessage$mcJJI$sp",
"com.signalcollect.interfaces.SignalMessage$mcFFS$sp",
"com.signalcollect.interfaces.SignalMessage$mcZVJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJVC$sp",
"com.signalcollect.interfaces.SignalMessage",
"com.signalcollect.interfaces.SignalMessage$mcCFF$sp",
"com.signalcollect.interfaces.SignalMessage$mcSCZ$sp",
"com.signalcollect.interfaces.Coordinator$class",
"com.signalcollect.interfaces.SignalMessage$mcFZV$sp",
"com.signalcollect.interfaces.SignalMessage$mcFDZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDBD$sp",
"com.signalcollect.interfaces.SignalMessage$mcCVC$sp",
"com.signalcollect.interfaces.SignalMessage$mcJFV$sp",
"com.signalcollect.interfaces.SignalMessage$mcBBV$sp",
"com.signalcollect.interfaces.SignalMessage$mcZDZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVBF$sp",
"com.signalcollect.interfaces.SignalMessage$mcCSF$sp",
"com.signalcollect.interfaces.SignalMessage$mcSJI$sp",
"com.signalcollect.interfaces.SignalMessage$mcICC$sp",
"com.signalcollect.interfaces.SignalMessage$mcVIB$sp",
"com.signalcollect.interfaces.SignalMessage$mcDII$sp",
"com.signalcollect.interfaces.SignalMessage$mcVCV$sp",
"com.signalcollect.interfaces.SignalMessage$mcBID$sp",
"com.signalcollect.interfaces.SignalMessage$mcSIZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcIVI$sp",
"com.signalcollect.interfaces.SignalMessage$mcIFD$sp",
"com.signalcollect.interfaces.SignalMessage$mcIJF$sp",
"com.signalcollect.interfaces.SignalMessage$mcBSJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJJD$sp",
"com.signalcollect.interfaces.SignalMessage$mcBII$sp",
"com.signalcollect.interfaces.SignalMessage$mcDSF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFIB$sp",
"com.signalcollect.interfaces.SignalMessage$mcVJV$sp",
"com.signalcollect.interfaces.SignalMessage$mcBJI$sp",
"com.signalcollect.interfaces.Debug$",
"com.signalcollect.interfaces.SignalMessage$mcDZF$sp",
"com.signalcollect.interfaces.SignalMessage$mcVSF$sp",
"com.signalcollect.interfaces.SignalMessage$mcZBZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDIC$sp",
"com.signalcollect.interfaces.SignalMessage$mcFFZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBDB$sp",
"com.signalcollect.interfaces.SignalMessage$mcFCZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcIZF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJDB$sp",
"com.signalcollect.interfaces.SignalMessage$mcJSI$sp",
"com.signalcollect.interfaces.WorkerStatistics",
"com.signalcollect.interfaces.SignalMessage$mcBCZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDSD$sp",
"com.signalcollect.interfaces.SignalMessage$mcZVC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDDJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVDZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDZS$sp",
"com.signalcollect.interfaces.SignalMessage$mcJIZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJBF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJIC$sp",
"com.signalcollect.interfaces.SignalMessage$mcFVV$sp",
"com.signalcollect.interfaces.SignalMessage$mcICV$sp",
"com.signalcollect.interfaces.SignalMessage$mcDDS$sp",
"com.signalcollect.interfaces.SignalMessage$mcVZV$sp",
"com.signalcollect.interfaces.SignalMessage$mcFFJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCJZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSID$sp",
"com.signalcollect.interfaces.Info",
"com.signalcollect.interfaces.SignalMessage$mcFCC$sp",
"com.signalcollect.interfaces.SignalMessage$mcSZS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIDI$sp",
"com.signalcollect.interfaces.SignalMessage$mcDZJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBIF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJDF$sp",
"com.signalcollect.interfaces.SignalMessage$mcZZI$sp",
"com.signalcollect.interfaces.SignalMessage$mcCZJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBSF$sp",
"com.signalcollect.interfaces.WorkerStatus$",
"com.signalcollect.interfaces.SignalMessage$mcIJB$sp",
"com.signalcollect.interfaces.SignalMessage$mcFZS$sp",
"com.signalcollect.interfaces.SignalMessage$mcCCZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSJC$sp",
"com.signalcollect.interfaces.SignalMessage$mcZIC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDFJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBVD$sp",
"com.signalcollect.interfaces.SignalMessage$mcVJC$sp",
"com.signalcollect.interfaces.SignalMessage$mcSFI$sp",
"com.signalcollect.interfaces.SignalMessage$mcZJB$sp",
"com.signalcollect.interfaces.Debug",
"com.signalcollect.interfaces.SignalMessage$mcCCB$sp",
"com.signalcollect.interfaces.SignalMessage$mcIJZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBIJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBSD$sp",
"com.signalcollect.interfaces.SignalMessage$mcJJZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZFV$sp",
"com.signalcollect.interfaces.StorageFactory",
"com.signalcollect.interfaces.SignalMessage$mcSVV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVVZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCVD$sp",
"com.signalcollect.interfaces.SignalMessage$mcCDD$sp",
"com.signalcollect.interfaces.SignalMessage$mcVZC$sp",
"com.signalcollect.interfaces.SignalMessage$mcVSJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCDJ$sp",
"com.signalcollect.interfaces.SignalMessage$",
"com.signalcollect.interfaces.SignalMessage$mcFZI$sp",
"com.signalcollect.interfaces.SignalMessage$mcCII$sp",
"com.signalcollect.interfaces.SignalMessage$mcFDS$sp",
"com.signalcollect.interfaces.SignalMessage$mcFJZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFJD$sp",
"com.signalcollect.interfaces.SignalMessage$mcIBB$sp",
"com.signalcollect.interfaces.SignalMessage$mcFCJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFJJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcICD$sp",
"com.signalcollect.interfaces.Factory$class",
"com.signalcollect.interfaces.SignalMessage$mcFBS$sp",
"com.signalcollect.interfaces.SignalMessage$mcSDJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJFC$sp",
"com.signalcollect.interfaces.SignalMessage$mcFBJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJBI$sp",
"com.signalcollect.interfaces.SignalMessage$mcBJV$sp",
"com.signalcollect.interfaces.SignalMessage$mcCVF$sp",
"com.signalcollect.interfaces.SignalMessage$mcDSJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBBF$sp",
"com.signalcollect.interfaces.SignalMessage$mcDIB$sp",
"com.signalcollect.interfaces.SignalMessage$mcSVJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVFJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCFJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSDC$sp",
"com.signalcollect.interfaces.SignalMessage$mcJDJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBJD$sp",
"com.signalcollect.interfaces.SignalMessage$mcBSS$sp",
"com.signalcollect.interfaces.SignalMessage$mcCIZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCJI$sp",
"com.signalcollect.interfaces.SignalMessage$mcSFB$sp",
"com.signalcollect.interfaces.SignalMessage$mcIZV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVJB$sp",
"com.signalcollect.interfaces.SignalMessage$mcSCB$sp",
"com.signalcollect.interfaces.SignalMessage$mcCCS$sp",
"com.signalcollect.interfaces.SignalMessage$mcDBI$sp",
"com.signalcollect.interfaces.SignalMessage$mcFBF$sp",
"com.signalcollect.interfaces.SignalMessage$mcBDZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZZC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDJJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSCV$sp",
"com.signalcollect.interfaces.SignalMessage$mcBVC$sp",
"com.signalcollect.interfaces.SignalMessage$mcSZF$sp",
"com.signalcollect.interfaces.SignalMessage$mcZFB$sp",
"com.signalcollect.interfaces.SignalMessage$mcZIF$sp",
"com.signalcollect.interfaces.SignalMessage$mcCBF$sp",
"com.signalcollect.interfaces.SignalMessage$mcZDS$sp",
"com.signalcollect.interfaces.SignalMessage$mcSDS$sp",
"com.signalcollect.interfaces.SignalMessage$mcZJD$sp",
"com.signalcollect.interfaces.SignalMessage$mcIFS$sp",
"com.signalcollect.interfaces.SignalMessage$mcSJF$sp",
"com.signalcollect.interfaces.SignalMessage$mcSZZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDVB$sp",
"com.signalcollect.interfaces.SignalMessage$mcISF$sp",
"com.signalcollect.interfaces.SignalMessage$mcDIF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFDC$sp",
"com.signalcollect.interfaces.SignalMessage$mcVCS$sp",
"com.signalcollect.interfaces.SignalMessage$mcVDC$sp",
"com.signalcollect.interfaces.SignalMessage$mcIIV$sp",
"com.signalcollect.interfaces.SignalMessage$mcFCF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJSV$sp",
"com.signalcollect.interfaces.SignalMessage$mcZDV$sp",
"com.signalcollect.interfaces.SignalMessage$mcIIJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcIBV$sp",
"com.signalcollect.interfaces.SignalMessage$mcZDI$sp",
"com.signalcollect.interfaces.Coordinator",
"com.signalcollect.interfaces.SignalMessage$mcZIZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBBC$sp",
"com.signalcollect.interfaces.SignalMessage$mcFVI$sp",
"com.signalcollect.interfaces.SignalMessage$mcFFC$sp",
"com.signalcollect.interfaces.SignalMessage$mcFSB$sp",
"com.signalcollect.interfaces.SignalMessage$mcIDZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSVC$sp",
"com.signalcollect.interfaces.Warning$",
"com.signalcollect.interfaces.SignalMessage$mcSJJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCCC$sp",
"com.signalcollect.interfaces.SignalMessage$mcCIF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJSD$sp",
"com.signalcollect.interfaces.SignalMessage$mcFVZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSDD$sp",
"com.signalcollect.interfaces.SignalMessage$mcBIB$sp",
"com.signalcollect.interfaces.SignalMessage$mcFBC$sp",
"com.signalcollect.interfaces.SignalMessage$mcIZI$sp",
"com.signalcollect.interfaces.SignalMessage$mcCZD$sp",
"com.signalcollect.interfaces.SignalMessage$mcVVB$sp",
"com.signalcollect.interfaces.SignalMessage$mcISI$sp",
"com.signalcollect.interfaces.SignalMessage$mcBCC$sp",
"com.signalcollect.interfaces.Serializer",
"com.signalcollect.interfaces.SignalMessage$mcSVD$sp",
"com.signalcollect.interfaces.SignalMessage$mcDZB$sp",
"com.signalcollect.interfaces.SignalMessage$mcIJC$sp",
"com.signalcollect.interfaces.SignalMessage$mcVZS$sp",
"com.signalcollect.interfaces.SignalMessage$mcBZZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBJB$sp",
"com.signalcollect.interfaces.SignalMessage$mcFZZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcIVJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFSC$sp",
"com.signalcollect.interfaces.SignalMessage$mcSZI$sp",
"com.signalcollect.interfaces.SignalMessage$mcVDS$sp",
"com.signalcollect.interfaces.SignalMessage$mcJFD$sp",
"com.signalcollect.interfaces.SignalMessage$mcZSC$sp",
"com.signalcollect.interfaces.SignalMessage$mcZID$sp",
"com.signalcollect.interfaces.SignalMessage$mcCBV$sp",
"com.signalcollect.interfaces.Worker",
"com.signalcollect.interfaces.SignalMessage$mcCBD$sp",
"com.signalcollect.interfaces.SignalMessage$mcZBB$sp",
"com.signalcollect.interfaces.SignalMessage$mcVIJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVVS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIDB$sp",
"com.signalcollect.interfaces.SignalMessage$mcSBZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJZJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSCC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDVI$sp",
"com.signalcollect.interfaces.SignalMessage$mcZZF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFID$sp",
"com.signalcollect.interfaces.SignalMessage$mcBFC$sp",
"com.signalcollect.interfaces.SignalMessage$mcVFS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIID$sp",
"com.signalcollect.interfaces.SignalMessage$mcIIC$sp",
"com.signalcollect.interfaces.SignalMessage$mcBDF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFDB$sp",
"com.signalcollect.interfaces.SignalMessage$mcBFJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFZF$sp",
"com.signalcollect.interfaces.SignalMessage$mcICB$sp",
"com.signalcollect.interfaces.SignalMessage$mcBFB$sp",
"com.signalcollect.interfaces.SignalMessage$mcVCJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBVV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVFC$sp",
"com.signalcollect.interfaces.SignalMessage$mcBCD$sp",
"com.signalcollect.interfaces.SignalMessage$mcJJJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJCD$sp",
"com.signalcollect.interfaces.SignalMessage$mcCJC$sp",
"com.signalcollect.interfaces.SignalMessage$mcBDD$sp",
"com.signalcollect.interfaces.SignalMessage$mcFZB$sp",
"com.signalcollect.interfaces.SignalMessage$mcCIV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVFD$sp",
"com.signalcollect.interfaces.SignalMessage$mcFCD$sp",
"com.signalcollect.interfaces.SignalMessage$mcVVI$sp",
"com.signalcollect.interfaces.SignalMessage$mcZDC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDZZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSVI$sp",
"com.signalcollect.interfaces.SignalMessage$mcFCI$sp",
"com.signalcollect.interfaces.SignalMessage$mcJSS$sp",
"com.signalcollect.interfaces.SignalMessage$mcBCV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVZB$sp",
"com.signalcollect.interfaces.SignalMessage$mcBBI$sp",
"com.signalcollect.interfaces.SignalMessage$mcVJS$sp",
"com.signalcollect.interfaces.SignalMessage$mcBZV$sp",
"com.signalcollect.interfaces.SignalMessage$mcSIS$sp",
"com.signalcollect.interfaces.SignalMessage$mcZFI$sp",
"com.signalcollect.interfaces.SignalMessage$mcBVZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJSB$sp",
"com.signalcollect.interfaces.SignalMessage$mcBDJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSIV$sp",
"com.signalcollect.interfaces.SignalMessage$mcJDD$sp",
"com.signalcollect.interfaces.SignalMessage$mcJDZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZVD$sp",
"com.signalcollect.interfaces.SignalMessage$mcFBZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCBJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJIS$sp",
"com.signalcollect.interfaces.SignalMessage$mcZBI$sp",
"com.signalcollect.interfaces.SignalMessage$mcJZF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFDV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVSB$sp",
"com.signalcollect.interfaces.SignalMessage$mcJJC$sp",
"com.signalcollect.interfaces.SignalMessage$mcIDV$sp",
"com.signalcollect.interfaces.Worker$class",
"com.signalcollect.interfaces.SignalMessage$mcDJC$sp",
"com.signalcollect.interfaces.SignalMessage$mcFZJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJII$sp",
"com.signalcollect.interfaces.SignalMessage$mcDCF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJCC$sp",
"com.signalcollect.interfaces.SignalMessage$mcSZD$sp",
"com.signalcollect.interfaces.SignalMessage$mcSBF$sp",
"com.signalcollect.interfaces.SignalMessage$mcDDF$sp",
"com.signalcollect.interfaces.SignalMessage$mcVBI$sp",
"com.signalcollect.interfaces.SignalMessage$mcFSZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBJS$sp",
"com.signalcollect.interfaces.SignalMessage$mcDDB$sp",
"com.signalcollect.interfaces.SignalMessage$mcBIS$sp",
"com.signalcollect.interfaces.SignalMessage$mcSBJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVZF$sp",
"com.signalcollect.interfaces.SignalMessage$mcDCZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFJC$sp",
"com.signalcollect.interfaces.SignalMessage$mcZFC$sp",
"com.signalcollect.interfaces.SignalMessage$mcCFI$sp",
"com.signalcollect.interfaces.SignalMessage$mcIZB$sp",
"com.signalcollect.interfaces.SignalMessage$mcSIB$sp",
"com.signalcollect.interfaces.SignalMessage$mcIFZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDVV$sp",
"com.signalcollect.interfaces.SignalMessage$mcSBI$sp",
"com.signalcollect.interfaces.WorkerStatus",
"com.signalcollect.interfaces.SignalMessage$mcBCJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDDD$sp",
"com.signalcollect.interfaces.SignalMessage$mcSBD$sp",
"com.signalcollect.interfaces.SignalMessage$mcZVZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZVS$sp",
"com.signalcollect.interfaces.SignalMessage$mcDVS$sp",
"com.signalcollect.interfaces.SignalMessage$mcVJZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSFD$sp",
"com.signalcollect.interfaces.SignalMessage$mcVDJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBBJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCBI$sp",
"com.signalcollect.interfaces.SignalMessage$mcDFZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBDV$sp",
"com.signalcollect.interfaces.AggregationOperation",
"com.signalcollect.interfaces.SignalMessage$mcVBS$sp",
"com.signalcollect.interfaces.SignalMessage$mcJCZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZDB$sp",
"com.signalcollect.interfaces.SignalMessage$mcJBV$sp",
"com.signalcollect.interfaces.SignalMessage$mcCFS$sp",
"com.signalcollect.interfaces.SignalMessage$mcDBZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcIFB$sp",
"com.signalcollect.interfaces.SignalMessage$mcSVS$sp",
"com.signalcollect.interfaces.SignalMessage$mcBFD$sp",
"com.signalcollect.interfaces.SignalMessage$mcJSF$sp",
"com.signalcollect.interfaces.SignalMessage$mcVVJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZFF$sp",
"com.signalcollect.interfaces.SignalMessage$mcCDI$sp",
"com.signalcollect.interfaces.SignalMessage$mcCJJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVFB$sp",
"com.signalcollect.interfaces.SignalMessage$mcIDS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIJV$sp",
"com.signalcollect.interfaces.SignalMessage$mcZVV$sp",
"com.signalcollect.interfaces.MessageBus",
"com.signalcollect.interfaces.SignalMessage$mcFFF$sp",
"com.signalcollect.interfaces.SignalMessage$mcBBD$sp",
"com.signalcollect.interfaces.SignalMessage$mcSSD$sp",
"com.signalcollect.interfaces.SignalMessage$mcFFI$sp",
"com.signalcollect.interfaces.SignalMessage$mcDSZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcIZS$sp",
"com.signalcollect.interfaces.SignalMessage$mcBBZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcICJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVJF$sp",
"com.signalcollect.interfaces.SignalMessage$mcDJS$sp",
"com.signalcollect.interfaces.SignalMessage$mcFVF$sp",
"com.signalcollect.interfaces.SignalMessage$mcVDI$sp",
"com.signalcollect.interfaces.SignalMessage$mcFIJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVIS$sp",
"com.signalcollect.interfaces.SignalMessage$mcSFF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJIV$sp",
"com.signalcollect.interfaces.SignalMessage$mcZFZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVSD$sp",
"com.signalcollect.interfaces.VertexToWorkerMapper",
"com.signalcollect.interfaces.SignalMessage$mcCIC$sp",
"com.signalcollect.interfaces.SignalMessage$mcCID$sp",
"com.signalcollect.interfaces.SignalMessage$mcIIF$sp",
"com.signalcollect.interfaces.SignalMessage$mcCDF$sp",
"com.signalcollect.interfaces.SignalMessage$mcSBV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVBV$sp",
"com.signalcollect.interfaces.SignalMessage$mcCBZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJVI$sp",
"com.signalcollect.interfaces.SignalMessage$mcDJB$sp",
"com.signalcollect.interfaces.SignalMessage$mcCFV$sp",
"com.signalcollect.interfaces.SignalMessage$mcJFS$sp",
"com.signalcollect.interfaces.SignalMessage$mcFFB$sp",
"com.signalcollect.interfaces.SignalMessage$mcSIJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJZI$sp",
"com.signalcollect.interfaces.EXTERNAL",
"com.signalcollect.interfaces.SignalMessage$mcIII$sp",
"com.signalcollect.interfaces.SignalMessage$mcJVJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJCJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZZZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJJS$sp",
"com.signalcollect.interfaces.SignalMessage$mcVVC$sp",
"com.signalcollect.interfaces.SignalMessage$mcSSB$sp",
"com.signalcollect.interfaces.SignalMessage$mcVSZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJZD$sp",
"com.signalcollect.interfaces.WorkerStatistics$",
"com.signalcollect.interfaces.SignalMessage$mcJBD$sp",
"com.signalcollect.interfaces.SignalMessage$mcJFI$sp",
"com.signalcollect.interfaces.SignalMessage$mcSCS$sp",
"com.signalcollect.interfaces.SignalMessage$mcDJF$sp",
"com.signalcollect.interfaces.SignalMessage$mcBVB$sp",
"com.signalcollect.interfaces.SignalMessage$mcCSI$sp",
"com.signalcollect.interfaces.SignalMessage$mcCJF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFZC$sp",
"com.signalcollect.interfaces.SignalMessage$mcIBS$sp",
"com.signalcollect.interfaces.SignalMessage$mcCJS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIBD$sp",
"com.signalcollect.interfaces.SignalMessage$mcDVZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFIF$sp",
"com.signalcollect.interfaces.SignalMessage$mcIVV$sp",
"com.signalcollect.interfaces.SignalMessage$mcZJF$sp",
"com.signalcollect.interfaces.SignalMessage$mcDFD$sp",
"com.signalcollect.interfaces.SignalMessage$mcBZD$sp",
"com.signalcollect.interfaces.SignalMessage$mcZCF$sp",
"com.signalcollect.interfaces.SignalMessage$mcCSD$sp",
"com.signalcollect.interfaces.SignalMessage$mcCSJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBJZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBFZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcIZZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZCD$sp",
"com.signalcollect.interfaces.SignalMessage$mcVIZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZSJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDCJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZSD$sp",
"com.signalcollect.interfaces.SignalMessage$mcJZS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIDC$sp",
"com.signalcollect.interfaces.SignalMessage$mcZSZ$sp",
"com.signalcollect.interfaces.MessageBusFactory",
"com.signalcollect.interfaces.SignalMessage$mcCFD$sp",
"com.signalcollect.interfaces.SignalMessage$mcZBD$sp",
"com.signalcollect.interfaces.SignalMessage$mcIDJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcIIB$sp",
"com.signalcollect.interfaces.SignalMessage$mcVBC$sp",
"com.signalcollect.interfaces.SignalMessage$mcVCI$sp",
"com.signalcollect.interfaces.VertexIdSet",
"com.signalcollect.interfaces.SignalMessage$mcBCB$sp",
"com.signalcollect.interfaces.SignalMessage$mcFCS$sp",
"com.signalcollect.interfaces.SignalMessage$mcSSJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCJD$sp",
"com.signalcollect.interfaces.SignalMessage$mcZDF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFFD$sp",
"com.signalcollect.interfaces.SignalMessage$mcSSC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDSI$sp",
"com.signalcollect.interfaces.SignalMessage$mcVBB$sp",
"com.signalcollect.interfaces.SignalMessage$mcFIS$sp",
"com.signalcollect.interfaces.SignalMessage$mcCVJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDBJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcISD$sp",
"com.signalcollect.interfaces.SignalMessage$mcZSB$sp",
"com.signalcollect.interfaces.SignalMessage$mcDFF$sp",
"com.signalcollect.interfaces.SignalMessage$mcBBB$sp",
"com.signalcollect.interfaces.SignalMessage$mcFVJ$sp",
"com.signalcollect.interfaces.EXTERNAL$",
"com.signalcollect.interfaces.SignalMessage$mcVID$sp",
"com.signalcollect.interfaces.SignalMessage$mcBZC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDSS$sp",
"com.signalcollect.interfaces.SignalMessage$mcDIS$sp",
"com.signalcollect.interfaces.SignalMessage$mcVVF$sp",
"com.signalcollect.interfaces.SignalMessage$mcSVF$sp",
"com.signalcollect.interfaces.SignalMessage$mcIVD$sp",
"com.signalcollect.interfaces.SignalMessage$mcISC$sp",
"com.signalcollect.interfaces.SignalMessage$mcZSF$sp",
"com.signalcollect.interfaces.SignalMessage$mcBCS$sp",
"com.signalcollect.interfaces.SignalMessage$mcDID$sp",
"com.signalcollect.interfaces.SignalMessage$mcCFZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcIBC$sp",
"com.signalcollect.interfaces.SignalMessage$mcZSI$sp",
"com.signalcollect.interfaces.SignalMessage$mcBIZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDIZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSDF$sp",
"com.signalcollect.interfaces.LogMessage",
"com.signalcollect.interfaces.SignalMessage$mcJSJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFDJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZZB$sp",
"com.signalcollect.interfaces.SignalMessage$mcISS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIFJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFDI$sp",
"com.signalcollect.interfaces.SignalMessage$mcCBB$sp",
"com.signalcollect.interfaces.SignalMessage$mcBSI$sp",
"com.signalcollect.interfaces.SignalMessage$mcDVF$sp",
"com.signalcollect.interfaces.SignalMessage$mcBJC$sp",
"com.signalcollect.interfaces.SignalMessage$mcVFZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCBC$sp",
"com.signalcollect.interfaces.SignalMessage$mcSCJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSFS$sp",
"com.signalcollect.interfaces.SignalMessage$mcFSV$sp",
"com.signalcollect.interfaces.SignalMessage$mcSDZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVCB$sp",
"com.signalcollect.interfaces.SignalMessage$mcFII$sp",
"com.signalcollect.interfaces.SignalMessage$mcDSB$sp",
"com.signalcollect.interfaces.SignalMessage$mcSSF$sp",
"com.signalcollect.interfaces.SignalMessage$mcSSI$sp",
"com.signalcollect.interfaces.SignalMessage$mcSDB$sp",
"com.signalcollect.interfaces.SignalMessage$mcZZS$sp",
"com.signalcollect.interfaces.SignalMessage$mcSFJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCZF$sp",
"com.signalcollect.interfaces.SignalMessage$mcDFC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDFB$sp",
"com.signalcollect.interfaces.SignalMessage$mcDDI$sp",
"com.signalcollect.interfaces.Severe",
"com.signalcollect.interfaces.SignalMessage$mcZJC$sp",
"com.signalcollect.interfaces.SignalMessage$mcFVS$sp",
"com.signalcollect.interfaces.SignalMessage$mcBZB$sp",
"com.signalcollect.interfaces.SignalMessage$mcIBZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCZZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZCI$sp",
"com.signalcollect.interfaces.SignalMessage$mcFJI$sp",
"com.signalcollect.interfaces.SignalMessage$mcDCC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDVC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDBF$sp",
"com.signalcollect.interfaces.SignalMessage$mcDCD$sp",
"com.signalcollect.interfaces.SignalMessage$mcVJI$sp",
"com.signalcollect.interfaces.SignalMessage$mcDCI$sp",
"com.signalcollect.interfaces.SignalMessage$mcCVB$sp",
"com.signalcollect.interfaces.SignalMessage$mcICF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFVB$sp",
"com.signalcollect.interfaces.SignalMessage$mcSFV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVSS$sp",
"com.signalcollect.interfaces.VertexSignalBuffer$class",
"com.signalcollect.interfaces.SignalMessage$mcZFD$sp",
"com.signalcollect.interfaces.SignalMessage$mcFBV$sp",
"com.signalcollect.interfaces.SignalMessage$mcDBB$sp",
"com.signalcollect.interfaces.VertexSignalBuffer",
"com.signalcollect.interfaces.SignalMessage$mcIBJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDSV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVSC$sp",
"com.signalcollect.interfaces.SignalMessage$mcZIJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFVC$sp",
"com.signalcollect.interfaces.SignalMessage$mcJFJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZVI$sp",
"com.signalcollect.interfaces.SignalMessage$mcBVI$sp",
"com.signalcollect.interfaces.SignalMessage$mcDBC$sp",
"com.signalcollect.interfaces.SignalMessage$mcIVB$sp",
"com.signalcollect.interfaces.SignalMessage$mcVZI$sp",
"com.signalcollect.interfaces.SignalMessage$mcCCJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCCV$sp",
"com.signalcollect.interfaces.SignalMessage$mcBCI$sp",
"com.signalcollect.interfaces.SignalMessage$mcJZC$sp",
"com.signalcollect.interfaces.SignalMessage$mcJBC$sp",
"com.signalcollect.interfaces.SignalMessage$mcVCD$sp",
"com.signalcollect.interfaces.SignalMessage$mcJVZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBDS$sp",
"com.signalcollect.interfaces.SignalMessage$mcFSS$sp",
"com.signalcollect.interfaces.SignalMessage$mcJSZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBIC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDCS$sp",
"com.signalcollect.interfaces.SignalMessage$mcBVS$sp",
"com.signalcollect.interfaces.SignalMessage$mcSSS$sp",
"com.signalcollect.interfaces.SignalMessage$mcSCI$sp",
"com.signalcollect.interfaces.SignalMessage$mcJCI$sp",
"com.signalcollect.interfaces.SignalMessage$mcBJF$sp",
"com.signalcollect.interfaces.SignalMessage$mcIVC$sp",
"com.signalcollect.interfaces.SignalMessage$mcCDZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZZD$sp",
"com.signalcollect.interfaces.SignalMessage$mcVCZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVDB$sp",
"com.signalcollect.interfaces.SignalMessage$mcFCB$sp",
"com.signalcollect.interfaces.SignalMessage$mcBSV$sp",
"com.signalcollect.interfaces.SignalMessage$mcFBB$sp",
"com.signalcollect.interfaces.SignalMessage$mcCCF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFJF$sp",
"com.signalcollect.interfaces.SignalMessage$mcZVF$sp",
"com.signalcollect.interfaces.SignalMessage$mcSII$sp",
"com.signalcollect.interfaces.SignalMessage$mcBVF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJBZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJFZ$sp",
"com.signalcollect.interfaces.Logging",
"com.signalcollect.interfaces.SignalMessage$mcFVD$sp",
"com.signalcollect.interfaces.SignalMessage$mcSBB$sp",
"com.signalcollect.interfaces.SignalMessage$mcCDB$sp",
"com.signalcollect.interfaces.SignalMessage$mcVIF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFSD$sp",
"com.signalcollect.interfaces.SignalMessage$mcBBS$sp",
"com.signalcollect.interfaces.SignalMessage$mcCVS$sp",
"com.signalcollect.interfaces.SignalMessage$mcFBD$sp",
"com.signalcollect.interfaces.SignalMessage$mcDJV$sp",
"com.signalcollect.interfaces.SignalMessage$mcJFF$sp",
"com.signalcollect.interfaces.SignalMessage$mcSJS$sp",
"com.signalcollect.interfaces.SignalMessage$mcSJZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVSI$sp",
"com.signalcollect.interfaces.SignalMessage$mcZBJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCSV$sp",
"com.signalcollect.interfaces.SignalMessage$mcZZJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSDI$sp",
"com.signalcollect.interfaces.SignalMessage$mcSSV$sp",
"com.signalcollect.interfaces.SignalMessage$mcDZI$sp",
"com.signalcollect.interfaces.SignalMessage$mcIFV$sp",
"com.signalcollect.interfaces.SignalMessage$mcSIC$sp",
"com.signalcollect.interfaces.SignalMessage$mcBJJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVBZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVCF$sp",
"com.signalcollect.interfaces.SignalMessage$mcBSB$sp",
"com.signalcollect.interfaces.SignalMessage$mcVIV$sp",
"com.signalcollect.interfaces.SignalMessage$mcCIB$sp",
"com.signalcollect.interfaces.SignalMessage$mcDJZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFJV$sp",
"com.signalcollect.interfaces.SignalMessage$mcSSZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcICS$sp",
"com.signalcollect.interfaces.SignalMessage$mcCBS$sp",
"com.signalcollect.interfaces.SignalMessage$mcCZI$sp",
"com.signalcollect.interfaces.SignalMessage$mcJCS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIBF$sp",
"com.signalcollect.interfaces.SignalMessage$mcZII$sp",
"com.signalcollect.interfaces.SignalMessage$mcBVJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcICI$sp",
"com.signalcollect.interfaces.SignalMessage$mcCIJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFZD$sp",
"com.signalcollect.interfaces.SignalMessage$mcZSV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVSV$sp",
"com.signalcollect.interfaces.SignalMessage$mcFSI$sp",
"com.signalcollect.interfaces.SignalMessage$mcCSB$sp",
"com.signalcollect.interfaces.SignalMessage$mcDDZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcSZJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZCS$sp",
"com.signalcollect.interfaces.SignalMessage$mcDCV$sp",
"com.signalcollect.interfaces.SignalMessage$mcIJD$sp",
"com.signalcollect.interfaces.SignalMessage$mcSBC$sp",
"com.signalcollect.interfaces.SignalMessage$mcIVZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZBF$sp",
"com.signalcollect.interfaces.SignalMessage$mcVDD$sp",
"com.signalcollect.interfaces.SignalMessage$mcZIB$sp",
"com.signalcollect.interfaces.SignalMessage$mcBFV$sp",
"com.signalcollect.interfaces.SignalMessage$mcCDV$sp",
"com.signalcollect.interfaces.SignalMessage$mcCCD$sp",
"com.signalcollect.interfaces.SignalMessage$mcCDC$sp",
"com.signalcollect.interfaces.Logging$class",
"com.signalcollect.interfaces.SignalMessage$mcIVS$sp",
"com.signalcollect.interfaces.SignalMessage$mcZCJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcIFF$sp",
"com.signalcollect.interfaces.SignalMessage$mcDZC$sp",
"com.signalcollect.interfaces.SignalMessage$mcZJV$sp",
"com.signalcollect.interfaces.SignalMessage$mcZCC$sp",
"com.signalcollect.interfaces.SignalMessage$mcZJZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcVFV$sp",
"com.signalcollect.interfaces.SignalMessage$mcZZV$sp",
"com.signalcollect.interfaces.SignalMessage$mcJJF$sp",
"com.signalcollect.interfaces.SignalMessage$mcCZB$sp",
"com.signalcollect.interfaces.SignalMessage$mcSJB$sp",
"com.signalcollect.interfaces.SignalMessage$mcFIV$sp",
"com.signalcollect.interfaces.SignalMessage$mcVDF$sp",
"com.signalcollect.interfaces.SignalMessage$mcZCV$sp",
"com.signalcollect.interfaces.SignalMessage$mcBZF$sp",
"com.signalcollect.interfaces.SignalMessage$mcIBI$sp",
"com.signalcollect.interfaces.SignalMessage$mcZFJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZJS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIDD$sp",
"com.signalcollect.interfaces.SignalMessage$mcCCI$sp",
"com.signalcollect.interfaces.SignalMessage$mcJVD$sp",
"com.signalcollect.interfaces.Severe$",
"com.signalcollect.interfaces.SignalMessage$mcBFI$sp",
"com.signalcollect.interfaces.SignalMessage$mcJVB$sp",
"com.signalcollect.interfaces.SignalMessage$mcBFS$sp",
"com.signalcollect.interfaces.SignalMessage$mcZSS$sp",
"com.signalcollect.interfaces.SignalMessage$mcJVF$sp",
"com.signalcollect.interfaces.SignalMessage$mcSCD$sp",
"com.signalcollect.interfaces.SignalMessage$mcIDF$sp",
"com.signalcollect.interfaces.SignalMessage$mcBIV$sp",
"com.signalcollect.interfaces.SignalMessage$mcDZV$sp",
"com.signalcollect.interfaces.SignalMessage$mcISB$sp",
"com.signalcollect.interfaces.SignalMessage$mcBZS$sp",
"com.signalcollect.interfaces.SignalMessage$mcFIC$sp",
"com.signalcollect.interfaces.SignalMessage$mcCJB$sp",
"com.signalcollect.interfaces.SignalMessage$mcDFV$sp",
"com.signalcollect.interfaces.SignalMessage$mcZFS$sp",
"com.signalcollect.interfaces.SignalMessage$mcDDV$sp",
"com.signalcollect.interfaces.SignalMessage$mcJCB$sp",
"com.signalcollect.interfaces.SignalMessage$mcJIJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZJJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDDC$sp",
"com.signalcollect.interfaces.Info$",
"com.signalcollect.interfaces.VertexStore",
"com.signalcollect.interfaces.SignalMessage$mcCZS$sp",
"com.signalcollect.interfaces.SignalMessage$mcZCB$sp",
"com.signalcollect.interfaces.SignalMessage$mcIZD$sp",
"com.signalcollect.interfaces.SignalMessage$mcISV$sp",
"com.signalcollect.interfaces.SignalMessage$mcSZB$sp",
"com.signalcollect.interfaces.SignalMessage$mcDVJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZIS$sp",
"com.signalcollect.interfaces.SignalMessage$mcCJV$sp",
"com.signalcollect.interfaces.SignalMessage$mcZIV$sp",
"com.signalcollect.interfaces.SignalMessage$mcIVF$sp",
"com.signalcollect.interfaces.WorkerFactory",
"com.signalcollect.interfaces.SignalMessage$mcBDI$sp",
"com.signalcollect.interfaces.SignalMessage$mcZDD$sp",
"com.signalcollect.interfaces.SignalMessage$mcDBS$sp",
"com.signalcollect.interfaces.SignalMessage$mcVFF$sp",
"com.signalcollect.interfaces.SignalMessage$mcDVD$sp",
"com.signalcollect.interfaces.SignalMessage$mcSJD$sp",
"com.signalcollect.interfaces.Storage",
"com.signalcollect.interfaces.SignalMessage$mcCDS$sp",
"com.signalcollect.interfaces.SignalMessage$mcVII$sp",
"com.signalcollect.interfaces.SignalMessage$mcVIC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDFI$sp",
"com.signalcollect.interfaces.SignalMessage$mcFSJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZCZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCVZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJBB$sp",
"com.signalcollect.interfaces.SignalMessage$mcZJI$sp",
"com.signalcollect.interfaces.SignalMessage$mcSFC$sp",
"com.signalcollect.interfaces.MessageRecipientRegistry",
"com.signalcollect.interfaces.SignalMessage$mcIJI$sp",
"com.signalcollect.interfaces.SignalMessage$mcFIZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCSZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFSF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFCV$sp",
"com.signalcollect.interfaces.SignalMessage$mcIFI$sp",
"com.signalcollect.interfaces.SignalMessage$mcSIF$sp",
"com.signalcollect.interfaces.SignalMessage$mcFJB$sp",
"com.signalcollect.interfaces.SignalMessage$mcDZD$sp",
"com.signalcollect.interfaces.SignalMessage$mcSZC$sp",
"com.signalcollect.interfaces.SignalMessage$mcDIV$sp",
"com.signalcollect.interfaces.SignalMessage$mcJZV$sp",
"com.signalcollect.interfaces.SignalMessage$mcSJV$sp",
"com.signalcollect.interfaces.SignalMessage$mcDIJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcJFB$sp",
"com.signalcollect.interfaces.SignalMessage$mcCFB$sp",
"com.signalcollect.interfaces.SignalMessage$mcBZJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCIS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIZJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcZBS$sp",
"com.signalcollect.interfaces.SignalMessage$mcFDF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJIF$sp",
"com.signalcollect.interfaces.SignalMessage$mcJIB$sp",
"com.signalcollect.interfaces.SignalMessage$mcVCC$sp",
"com.signalcollect.interfaces.SignalMessage$mcZDJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcFFV$sp",
"com.signalcollect.interfaces.SignalMessage$mcCFC$sp",
"com.signalcollect.interfaces.SignalMessage$mcICZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcBCF$sp",
"com.signalcollect.interfaces.SignalMessage$mcVBJ$sp",
"com.signalcollect.interfaces.SignalMessage$mcDSC$sp",
"com.signalcollect.interfaces.SignalMessage$mcCZV$sp",
"com.signalcollect.interfaces.SignalMessage$mcDFS$sp",
"com.signalcollect.interfaces.SignalMessage$mcIIZ$sp",
"com.signalcollect.interfaces.SignalMessage$mcCVI$sp",
"com.signalcollect.interfaces.SignalMessage$mcVVD$sp",
"com.signalcollect.interfaces.SignalMessage$mcVJD$sp",
"com.signalcollect.interfaces.SignalMessage$mcFBI$sp",
"com.signalcollect.EdgeId$mcSS$sp$class",
"com.signalcollect.EdgeId$mcZZ$sp$class",
"com.signalcollect.DefaultEdge$mcVV$sp",
"com.signalcollect.EdgeId$mcFZ$sp",
"com.signalcollect.DefaultEdge$mcDS$sp",
"com.signalcollect.EdgeId$mcIJ$sp",
"com.signalcollect.DefaultEdge$mcFJ$sp",
"com.signalcollect.DefaultEdge$mcBC$sp",
"com.signalcollect.DefaultEdge$mcBD$sp",
"com.signalcollect.DataFlowVertex$mcVV$sp",
"com.signalcollect.EdgeId$mcVF$sp",
"com.signalcollect.DefaultEdge$mcFF$sp",
"com.signalcollect.EdgeId$mcJZ$sp",
"com.signalcollect.OptionalSignalEdge$",
"com.signalcollect.EdgeId$mcBB$sp",
"com.signalcollect.EdgeId$mcFV$sp",
"com.signalcollect.EdgeId$mcJB$sp",
"com.signalcollect.DefaultEdge$mcBI$sp",
"com.signalcollect.DefaultEdge$mcSV$sp",
"com.signalcollect.DataGraphVertex$mcFJ$sp",
"com.signalcollect.DataGraphVertex$mcZZ$sp",
"com.signalcollect.Edge",
"com.signalcollect.EdgeId$mcBI$sp$class",
"com.signalcollect.DataFlowVertex$mcBJ$sp",
"com.signalcollect.EdgeId$mcVS$sp$class",
"com.signalcollect.DataGraphVertex$mcVI$sp",
"com.signalcollect.DefaultEdge$mcJJ$sp",
"com.signalcollect.DefaultEdge$mcVB$sp",
"com.github.nethad.clustermeister.integration.sc07.PageRankVertex",
"com.github.nethad.clustermeister.integration.sc07.PageRankEdge",

"com.github.nethad.clustermeister.integration.sc07.PageRankVertex$",
"com.github.nethad.clustermeister.integration.sc07.LoadRequestCallable$",
"com.github.nethad.clustermeister.integration.sc07.NodeProvisionerCreator$",
"com.github.nethad.clustermeister.integration.sc07.LoadRequestScenario",
"com.github.nethad.clustermeister.integration.sc07.Scenario07",
"com.github.nethad.clustermeister.integration.sc07.NodeControllerBootstrap$",
//"com.github.nethad.clustermeister.integration.sc07.CMAkkaConfig$",
"com.github.nethad.clustermeister.integration.sc07.ClustermeisterNodeProvisioner$$anonfun$1",
"com.github.nethad.clustermeister.integration.sc07.ClustermeisterNodeProvisioner$$anonfun$getNodes$1",
"com.github.nethad.clustermeister.integration.sc07.ClustermeisterNodeProvisioner$$anonfun$2",
"com.github.nethad.clustermeister.integration.sc07.PageRankVertex",
"com.github.nethad.clustermeister.integration.sc07.PageRank",
"com.github.nethad.clustermeister.integration.sc07.PageRank$delayedInit$body",
"com.github.nethad.clustermeister.integration.sc07.Scenario07$",
"com.github.nethad.clustermeister.integration.sc07.PageRank$$anonfun$1",
"com.github.nethad.clustermeister.integration.sc07.NodeControllerCreator",
"com.github.nethad.clustermeister.integration.sc07.PageRank$",
"com.github.nethad.clustermeister.integration.sc07.LoadRequestScenario$$anonfun$1",
"com.github.nethad.clustermeister.integration.sc07.LoadRequestScenario$delayedInit$body",
"com.github.nethad.clustermeister.integration.sc07.NodeControllerBootstrap$$anonfun$preloadClasses$1",
"com.github.nethad.clustermeister.integration.sc07.JPPFClassloader",
"com.github.nethad.clustermeister.integration.sc07.LoadRequestScenario$$anonfun$2",
//"com.github.nethad.clustermeister.integration.sc07.CMAkkaConfig$$anonfun$configStr$1",
"com.github.nethad.clustermeister.integration.sc07.NodeControllerBootstrap",
"com.github.nethad.clustermeister.integration.sc07.NodeProvisionerCreator",
"com.github.nethad.clustermeister.integration.sc07.Scenario07$$anonfun$runScenario$1",
"com.github.nethad.clustermeister.integration.sc07.Scenario07$delayedInit$body",
"com.github.nethad.clustermeister.integration.sc07.LoadRequestScenario$",
"com.github.nethad.clustermeister.integration.sc07.LoadRequestScenario$$anonfun$3",
"com.github.nethad.clustermeister.integration.sc07.PageRankEdge",
"com.github.nethad.clustermeister.integration.sc07.LoadRequestCallable",
"com.github.nethad.clustermeister.integration.sc07.NodeControllerCreator$",
"com.github.nethad.clustermeister.integration.sc07.ClustermeisterNodeProvisioner",
//"com.github.nethad.clustermeister.integration.sc07.CMAkkaConfig",
"com.github.nethad.clustermeister.integration.sc07.NodeControllerBootstrap$$anonfun$3"
)
  
  def call: String = {

    if (preload) {
      preloadClasses
    }
    
    println("Starting with config: " + akkaConfig.entrySet())
    
//    new JPPFClassloader().doClassloading(clazzes)
//    val cl = getClass().getClassLoader()
//    println("cl instanceof JPPFCl = "+(cl.isInstanceOf[AbstractJPPFClassLoader]))
    val system = ActorSystem("SignalCollect", akkaConfig)
    val nodeControllerCreator = NodeControllerCreator(nodeId, nodeProvisionerAddress)
    val nodeController = system.actorOf(Props().withCreator(nodeControllerCreator.create), name = "NodeController" + nodeId)
    //system.awaitTermination
    "Done"
  }
  
  def preloadClasses() = {
    var counter = 0
    for (clazz <- clazzes) {
      //println("Before loading "+clazz)
      Class.forName(clazz)
      //println("After loading "+clazz)
      counter += 1
      if (counter%50 == 0) {
        println(counter + " classes loaded.")
      }
    }
  }
}
